﻿
---------------------------------------------------------
-- Addon declaration
---------------------------------------------------------
HandyNotes_SpecialNPCsEtc = LibStub("AceAddon-3.0"):NewAddon("HandyNotes_SpecialNPCsEtc","AceEvent-3.0")
local HNS = HandyNotes_SpecialNPCsEtc
local Astrolabe = DongleStub("Astrolabe-0.4")
local L = LibStub("AceLocale-3.0"):GetLocale("HandyNotes_SpecialNPCsEtc")


---------------------------------------------------------
-- Our db upvalue and db defaults
---------------------------------------------------------
local CURRENT_DB_VERSION = 2   -- added npcID
local db
local defaults = {
	profile = {
		icon_scale = 1.0,
		icon_alpha = 1.0,
	},
	factionrealm = {
		dbversion = 0,
		nodes = {
			["*"] = {},
		}
	},
}


---------------------------------------------------------
-- Localize some globals
---------------------------------------------------------
local next = next
local select = select
local string_find = string.find
local GameTooltip = GameTooltip
local WorldMapTooltip = WorldMapTooltip
local HandyNotes = HandyNotes


---------------------------------------------------------
-- Constants
---------------------------------------------------------
local iconpath = "Interface\\AddOns\\HandyNotes_Trainers\\Artwork\\"

local defkey = {}
local iconDB = {

	-- Classes
	["DRUID"]   = iconpath .. "Druid",
	["HUNTER"]  = iconpath .. "Hunter",
	["MAGE"]    = iconpath .. "Mage",
	["PALADIN"] = iconpath .. "Paladin",
	["PRIEST"]  = iconpath .. "Priest",
	["ROGUE"]   = iconpath .. "Rogue",
	["SHAMAN"]  = iconpath .. "Shaman",
	["WARLOCK"] = iconpath .. "Warlock",
	["WARRIOR"] = iconpath .. "Warrior",
	["DEATHKNIGHT"]  = iconpath .. "Deathknight",   -- TODO: Check the key from WotLK

	-- Primary
	["Alchemy"]        = iconpath .. "Alchemy",
	["Blacksmithing"]  = iconpath .. "Blacksmithing",
	["Armorsmith"]     = iconpath .. "Blacksmithing",
	["Weaponsmith"]    = iconpath .. "Blacksmithing",
	["Enchanting"]     = iconpath .. "Enchanting",
	["Engineering"]    = iconpath .. "Engineering",
	["Herbalism"]      = iconpath .. "Herbalism",
	["Inscription"]    = iconpath .. "Inscription",
	["Jewelcrafting"]  = iconpath .. "Jewelcrafting",
	["Leatherworking"] = iconpath .. "Leatherworking",
	["Mining"]         = iconpath .. "Mining",
	["Skinning"]       = iconpath .. "Skinning",
	["Tailoring"]      = iconpath .. "Tailoring",

	-- Secondary
	["Cooking"]        = iconpath .. "Cooking",
	["First Aid"]      = iconpath .. "Firstaid",
	["Fishing"]        = iconpath .. "Fishing",

	-- Special
	["WeaponMaster"]   = iconpath .. "Weaponmaster",
	["Riding"]         = iconpath .. "Riding",
	["ColdFlying"]     = iconpath .. "Riding",

	["Portal"]  = iconpath .. "Portal",

	["Pet"]     = iconpath .. "Pet",
	
	["Innkeeper"] = "Interface\\Minimap\\Tracking\\Innkeeper",
	["Auctioneer"] = "Interface\\Minimap\\Tracking\\Auctioneer",
	["Banker"] = "Interface\\Minimap\\Tracking\\Banker",
	["Battlemaster"] = "Interface\\Minimap\\Tracking\\BattleMaster",
	["Stable"] = "Interface\\Minimap\\Tracking\\StableMaster",
	["SpiritHealer"] = "Interface\\Minimap\\Tracking\\TrivialQuests",
	["GuildVault"] = "Interface\\GLUES\\LOGIN\\Glues-CheckBox-Check",
	
	-- Default
	[defkey]    = "Interface\\Minimap\\Tracking\\TrivialQuests", -- for any SpecialNPCsEtc without icon definition
}

setmetatable(iconDB, {__index = function (t, k)
					local v = t[defkey]
					rawset(t, k, v) -- cache the value for next retrievals
					return v
				end})

---------------------------------------------------------
-- Plugin Handlers to HandyNotes
---------------------------------------------------------

local HNSHandler = {}

local function deletePin(button, mapFile, coord)
	local x, y = HandyNotes:getXY(coord)
	db.factionrealm.nodes[mapFile][coord] = nil
	HNS:SendMessage("HandyNotes_NotifyUpdate", "SpecialNPCsEtc")
end

local function createWaypoint(button, mapFile, coord)
	local c, z = HandyNotes:GetCZ(mapFile)
	local x, y = HandyNotes:getXY(coord)
	local vType, vName, vGuild = strsplit(":", db.factionrealm.nodes[mapFile][coord])
	if TomTom then
		TomTom:AddZWaypoint(c, z, x*100, y*100, vName)
	elseif Cartographer_Waypoints then
		Cartographer_Waypoints:AddWaypoint(NotePoint:new(HandyNotes:GetCZToZone(c, z), x, y, vName))
	end
end

local clickedNote, clickedNoteZone
local info = {}
local function generateMenu(button, level)
	if (not level) then return end
	for k in pairs(info) do info[k] = nil end
	if (level == 1) then
		-- Create the title of the menu
		info.isTitle      = 1
		info.text         = L["HandyNotes - SpecialNPCsEtc"]
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)

		if TomTom or Cartographer_Waypoints then
			-- Waypoint menu item
			info.disabled     = nil
			info.isTitle      = nil
			info.notCheckable = nil
			info.text = L["Create waypoint"]
			info.icon = nil
			info.func = createWaypoint
			info.arg1 = clickedNoteZone
			info.arg2 = clickedNote
			UIDropDownMenu_AddButton(info, level);
		end

		-- Delete menu item
		info.disabled     = nil
		info.isTitle      = nil
		info.notCheckable = nil
		info.text = L["Delete SpecialNPCsEtc"]
		info.icon = nil
		info.func = deletePin
		info.arg1 = clickedNoteZone
		info.arg2 = clickedNote
		UIDropDownMenu_AddButton(info, level);

		-- Close menu item
		info.text         = L["Close"]
		info.icon         = nil
		info.func         = function() CloseDropDownMenus() end
		info.arg1         = nil
		info.arg2         = nil
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level);
	end
end
local HNS_Dropdown = CreateFrame("Frame", "HandyNotes_SpecialNPCsEtcDropdownMenu")
HNS_Dropdown.displayMode = "MENU"
HNS_Dropdown.initialize = generateMenu

function HNSHandler:OnClick(button, down, mapFile, coord)
	if button == "RightButton" and not down then
		clickedNoteZone = mapFile
		clickedNote = coord
		ToggleDropDownMenu(1, nil, HNS_Dropdown, self, 0, 0)
	end
end

function HNSHandler:OnEnter(mapFile, coord)
	local tooltip = self:GetParent() == WorldMapButton and WorldMapTooltip or GameTooltip
	if ( self:GetCenter() > UIParent:GetCenter() ) then -- compare X coordinate
		tooltip:SetOwner(self, "ANCHOR_LEFT")
	else
		tooltip:SetOwner(self, "ANCHOR_RIGHT")
	end
	local vType, vName, vGuild = strsplit(":", db.factionrealm.nodes[mapFile][coord])
	tooltip:AddLine("|cffe0e0e0"..vName.."|r")
	if (vGuild ~= "") then tooltip:AddLine(vGuild) end
	tooltip:Show()
end

function HNSHandler:OnLeave(mapFile, coord)
	if self:GetParent() == WorldMapButton then
		WorldMapTooltip:Hide()
	else
		GameTooltip:Hide()
	end
end

do
	-- This is a custom iterator we use to iterate over every node in a given zone
	local function iter(t, prestate)
		if not t then return nil end
		local state, value = next(t, prestate)
		while state do
			if value then
				local vType, vName, vGuild = strsplit(":", value)
				local icon = iconDB[vType]
				return state, nil, icon, db.profile.icon_scale, db.profile.icon_alpha
			end
			state, value = next(t, state)
		end
		return nil, nil, nil, nil
	end
	function HNSHandler:GetNodes(mapFile)
		return iter, db.factionrealm.nodes[mapFile], nil
	end
end


---------------------------------------------------------
-- Options table
---------------------------------------------------------

local options = {
	type = "group",
	name = "SpecialNPCsEtc",
	desc = "SpecialNPCsEtc",
	get = function(info) return db.profile[info.arg] end,
	set = function(info, v)
		db.profile[info.arg] = v
		HNS:SendMessage("HandyNotes_NotifyUpdate", "SpecialNPCsEtc")
	end,
	args = {
		desc = {
			name = L["These settings control the look and feel of the SpecialNPCsEtc icons."],
			type = "description",
			order = 0,
		},
		icon_scale = {
			type = "range",
			name = L["Icon Scale"],
			desc = L["The scale of the icons"],
			min = 0.25, max = 2, step = 0.01,
			arg = "icon_scale",
			order = 10,
		},
		icon_alpha = {
			type = "range",
			name = L["Icon Alpha"],
			desc = L["The alpha transparency of the icons"],
			min = 0, max = 1, step = 0.01,
			arg = "icon_alpha",
			order = 20,
		},
	},
}


---------------------------------------------------------
-- NPC info tracking - TT handling
---------------------------------------------------------

local tt = CreateFrame("GameTooltip")
tt:SetOwner(UIParent, "ANCHOR_NONE")
tt.left = {}
tt.right = {}

for i = 1, 30 do
	tt.left[i] = tt:CreateFontString()
	tt.left[i]:SetFontObject(GameFontNormal)
	tt.right[i] = tt:CreateFontString()
	tt.right[i]:SetFontObject(GameFontNormal)
	tt:AddFontStrings(tt.left[i], tt.right[i])
end


local LEVEL_start = "^" .. (type(LEVEL) == "string" and LEVEL or "Level")
local function FigureNPCGuild(unit)
	tt:ClearLines()
	tt:SetUnit(unit)
	if not tt:IsOwned(UIParent) then
		tt:SetOwner(UIParent, "ANCHOR_NONE")
	end
	local left_2 = tt.left[2]:GetText()
	if not left_2 or left_2:find(LEVEL_start) then
		return ""
	end
	return left_2
end

---------------------------------------------------------
-- Addon initialization, enabling and disabling
---------------------------------------------------------

function HNS:OnInitialize()
	-- Set up our database
	db = LibStub("AceDB-3.0"):New("HandyNotes_SpecialNPCsEtcDB", defaults)
	self.db = db

	if db.factionrealm.dbversion > CURRENT_DB_VERSION then
		print("|cff6fafffHandyNotes_SpecialNPCsEtc:|r |cffff4f00Warning:|r Unknown database version. Please update to newer version.")
		print("|cff6fafffHandyNotes_SpecialNPCsEtc:|r |cffff4f00Warning:|r Addon has been disabled to protect your database.")
		self:Disable()
		return
	end

	if db.factionrealm.dbversion ~= CURRENT_DB_VERSION then
		if db.factionrealm.dbversion == 0 then 
			-- addon was just installed or using pre-versioned DB 
			if next(db.factionrealm.nodes, nil) then -- DB not empty -> first version
				db.factionrealm.dbversion = 1
			else
				db.factionrealm.dbversion = CURRENT_DB_VERSION 
			end
		end

		if db.factionrealm.dbversion == 1 then
			db.factionrealm.dbversion = 2 -- just adds npcID to the end of info string
		end
	end

	-- Initialize our database with HandyNotes
	HandyNotes:RegisterPluginDB("SpecialNPCsEtc", HNSHandler, options)
end

function HNS:OnEnable()
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("AUCTION_HOUSE_SHOW")
	self:RegisterEvent("GUILDBANKFRAME_OPENED")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("BATTLEFIELDS_SHOW")
	self:RegisterEvent("PET_STABLE_SHOW")
	self:RegisterEvent("CONFIRM_XP_LOSS")
	self:RegisterEvent("TRAINER_SHOW")
end


function HNS:TRAINER_SHOW()
	ChatFrame4:AddMessage("HNS:TRAINER_SHO");
	self:Opened_NPC("TRAINER_SHOW");
end

function HNS:AUCTION_HOUSE_SHOW()
	ChatFrame4:AddMessage("HNS:AUCTION_HOUSE_SHOW");
	self:Opened_NPC(L["Auctioneer"]);

end

function HNS:BANKFRAME_OPENED()
	ChatFrame4:AddMessage("HNS:BANKFRAME_OPENED");
	self:Opened_NPC(L["Banker"]);
end

function HNS:GUILDBANKFRAME_OPENED()
	ChatFrame4:AddMessage("HNS:GUILDBANKFRAME_OPENED");
	local vNpcid = 0; -- As this is not a real NPC or person then they don't have a NPC ID
	local vName = GetRealZoneText();
	local vGuild = "Guild Vault";
	local vType = "GuildVault";
	self:AddSpecialNPCsEtcNote(vNpcid, vName, vGuild, vType);
end

function HNS:BATTLEFIELDS_SHOW()
	ChatFrame4:AddMessage("HNS:BATTLEFIELDS_SHOW");
	self:Opened_NPC(L["Battlemaster"]);
end

function HNS:PET_STABLE_SHOW()
	ChatFrame4:AddMessage("HNS:PET_STABLE_SHOW");
	self:Opened_NPC(L["Stable Master"]);

end

function HNS:CONFIRM_XP_LOSS()
	ChatFrame4:AddMessage("HNS:CONFIRM_XP_LOSS");
	self:Opened_NPC(L["SpiritHealer"]);
end

function HNS:GOSSIP_SHOW()
	ChatFrame4:AddMessage("HNS:GOSSIP_SHOW");
	self:Opened_NPC();
end

local filters = {"available", "unavailable", "used"}
local filtersrestore = {}

function HNS:Opened_NPC(sType)
	ChatFrame4:AddMessage("HNS:Opened_NPC");
	local vName = UnitName("npc")
	local vGuild = FigureNPCGuild("npc")
	local vType = nil
	
	local vGUID = UnitGUID("npc")
	local vNpcid = tonumber(vGUID:sub(-12, -7), 16)
	
	if ((vGuild == nil or strlen(vGuild) == 0) and (sType ~= nil or strlen(sType) > 0)) then
		vGuild = sType;
	end
	
	if (sType == "TRAINER_SHOW") and IsTradeskillTrainer() then
		for i,f in ipairs(filters) do
			if GetTrainerServiceTypeFilter(f) ~= 1 then
				SetTrainerServiceTypeFilter(f, 1)
				filtersrestore[f] = true
			else
				filtersrestore[f] = nil
			end
		end

		ExpandTrainerSkillLine(1)  -- TODO: Find out, if the previous state can be saved

		vType = GetTrainerServiceSkillLine(2)

		for f,v in pairs(filtersrestore) do
			if v then
				SetTrainerServiceTypeFilter(f, 0)
			end
		end
	elseif string_find(vGuild, L["Weapon Master"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - WeaponMaster");
		vType = "WeaponMaster"
	elseif string_find(vGuild, L["Portal Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Portal");
		vType = "Portal"
	elseif string_find(vGuild, L["Pet Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Pet");
		vType = "Pet"
	elseif string_find(vGuild, L["Riding Trainer"]) or string_find(vGuild, L["Mechanostrider Pilot"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Riding");
		vType = "Riding"
	elseif string_find(vGuild, L["Cold Weather Flying Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - ColdFlying");
		vType = "ColdFlying"
		
	-- These are for when you don't have the ablity to learn this skill so it adds it anyway.
	elseif string_find(vGuild, L["Paladin Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - PALADIN");
		vType = "PALADIN"
	elseif string_find(vGuild, L["Mage Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - MAGE");
		vType = "MAGE"
	elseif string_find(vGuild, L["Druid Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - DRUID");
		vType = "DRUID"
	elseif string_find(vGuild, L["Hunter Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - HUNTER");
		vType = "HUNTER"
	elseif string_find(vGuild, L["Priest Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - PRIEST");
		vType = "PRIEST"
	elseif string_find(vGuild, L["Rogue Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - ROGUE");
		vType = "ROGUE"
	elseif string_find(vGuild, L["Sharman Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - SHAMAN");
		vType = "SHAMAN"
	elseif string_find(vGuild, L["Warlock Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - WARLOCK");
		vType = "WARLOCK"
	elseif string_find(vGuild, L["Warrior Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - WARRIOR");
		vType = "WARRIOR"
	elseif string_find(vGuild, L["Deathknight Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - DEATHKNIGHT");
		vType = "DEATHKNIGHT"
		
		
		
	elseif string_find(vGuild, L["Mining Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Mining Trainer");
		vType = "Mining"
	elseif string_find(vGuild, L["Armorsmith"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Armorsmith");
		vType = "Armorsmith"
	elseif string_find(vGuild, L["Blacksmithing Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Blacksmithing");
		vType = "Blacksmithing"
	elseif string_find(vGuild, L["Weaponsmith Trainer"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Blacksmithing");
		vType = "Weaponsmith"
	elseif string_find(vGuild, L["Axesmithing"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Blacksmithing");
		vType = "Axesmithing"
	elseif string_find(vGuild, L["Hammersmithing"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Hammersmithing");
		vType = "Hammersmithing"
	elseif string_find(vGuild, L["Swordsmithing"]) then
		ChatFrame4:AddMessage("HT:Opened_NPC() - Swordsmithing");
		vType = "Swordsmithing"
		
		
		
	-- Non-Trainer NPCs
	elseif string_find(vGuild, L["Innkeeper"]) then
	ChatFrame4:AddMessage("HNS:Opened_NPC - Innkeeper");
		vType = "Innkeeper"
	elseif string_find(vGuild, L["Auctioneer"]) then
	ChatFrame4:AddMessage("HNS:Opened_NPC - Auctioneer");
		vType = "Auctioneer"
	elseif string_find(vGuild, L["Banker"]) then
	ChatFrame4:AddMessage("HNS:Opened_NPC - Banker");
		vType = "Banker"
	elseif string_find(vGuild, L["Stable Master"]) then
	ChatFrame4:AddMessage("HNS:Opened_NPC - Stable");
		vType = "Stable"
	elseif string_find(vGuild, L["Battlemaster"]) then
	ChatFrame4:AddMessage("HNS:Opened_NPC - Battlemaster");
		vType = "Battlemaster"
	end
	
	if vType then
		ChatFrame4:AddMessage("HNS:Opened_NPC() - vNpcid = "..tostring(vNpcid)..", vName = "..tostring(vName)..", vGuild = "..tostring(vGuild)..", vType = "..tostring(vType));
		self:AddSpecialNPCsEtcNote(vNpcid, vName, vGuild, vType)
	end
end

local thres = 5 -- in yards
function HNS:AddSpecialNPCsEtcNote(vNpcid, vName, vGuild, vType)
	ChatFrame4:AddMessage("HNS:AddSpecialNPCsEtcNote: "..tostring(vName));
	local continent, zone, x, y = Astrolabe:GetCurrentPlayerPosition()
	
	if not vName or not continent then
		ChatFrame4:AddMessage("HNS:AddSpecialNPCsEtcNote: not vName or not continent");
		return
	end

	local vInfo1 = vType .. ":" .. vName .. ":" .. vGuild
	local vInfo2 = vType .. ":" .. vName .. ":" .. vGuild .. ":" .. vNpcid

	local coord = HandyNotes:getCoord(x, y)
	local map = HandyNotes:GetMapFile(continent, zone)
	if map then
		for coords, name in pairs(db.factionrealm.nodes[map]) do
			if vInfo1 == name or vInfo2 == name then
				local cx, cy = HandyNotes:getXY(coords)
				local dist = Astrolabe:ComputeDistance(continent, zone, x, y, continent, zone, cx, cy)
				if dist <= thres then -- SpecialNPCsEtc already exists here
					coord = coords  -- let's not move, just update vInfo
					-- return
				elseif (vNpcid > 0) then -- If this is def an NPC and the SpecialNPCsEtc exists on different location = has moved -> delete old info
					db.factionrealm.nodes[map][coords] = nil
				end
			end
		end
		
		ChatFrame4:AddMessage("vInfo2 = "..tostring(vInfo2));
		
		db.factionrealm.nodes[map][coord] = vInfo2
		self:SendMessage("HandyNotes_NotifyUpdate", "SpecialNPCsEtc")
	end
end
